// *********************************************************
//  ft_parInt_iif                            bach 05.09.2014
//                                              getestet OK! 
// ---------------------------------------------------------
//  1.1 Einstellung des Interfaces
//  Baudrate        9600
//  Bit             8
//  Parity          no
//  Stopbits        1
//
//  1.2 Kommunikation
//  Zur Ansteuerung des Interface sind grundstzlich zwei 
//  Bytes an das serielle Interface zu schicken. Das erste 
//  Byte ist der Interfacebefehl und das zweite Byte enthlt 
//  den Motorzustand.
//  Der Motorzustand beschreibt, welche Motoren in welcher 
//  Richtung gestartet werden sollen. Je nach Interfacebefehl 
//  antwortet das Interface mit einem oder drei Bytes.
//
//  1. Byte: Interfacebefehl
//  Binr    hex dezimal Beschreibung
//  11000001 C1  193     nur E/A-Zustand
//  11000101 C5  197     E/A-Zustand und Analogwert EX
//  11001001 C9  201     E/A-Zustand und Analogwert EY
//
//  2. Byte: Motorzustand
//  Bit  Beschreibung
//  1    Motor 4 links
//  2    Motor 4 rechts
//  3    Motor 3 links
//  4    Motor 3 rechts
//  5    Motor 2 links
//  6    Motor 2 rechts
//  7    Motor 1 links
//  8    Motor 1 rechts
//
//  Wird das linke und rechte Bit eines Motors gleichzeitig
//  gesetzt, so luft der Motor nicht (Stopp).
//
//  1.3 Antworten des Interfaces:
//  Befehl Antwort   Beschreibung
//  193    1 Byte    jedes Bit entspricht dem Wert des Eingangs
//  197    3 Bytes   1. Byte s.o. 2. und 3. Byte Analoginput EX
//  201    3 Bytes   1. Byte s.o. 2. und 3. Byte Analoginput EY
//
//  Wenn das Interface nicht ca. alle 200 ms einen Steuerbefehl 
//  erhlt, so schalten die Motoren sich selbstndig aus.
//
// *********************************************************
#include <ft_parInt.h>
#zero_ram

byte i, st[4];                             // uart buffer
byte rpos=0, wpos=0;                       // buffer pointer
int16 EX, EY;
#byte loEX=EX
#byte hiEX=EX+1
#byte loEY=EY
#byte hiEY=EY+1

// **********************************************************
byte read_uart (void) {
   if (rpos!=wpos) rpos=(rpos+1)&0x03;
return st[rpos];
}

// **********************************************************
void main() {
   setup_adc( ADC_CLOCK_INTERNAL );
   setup_adc_ports( EX_EY_analog ) ;
   set_adc_channel( _EX );
   delay_ms (5);
   enable_interrupts(INT_EXT1_H2L);
   enable_interrupts(GLOBAL);
/*
   printf ("\r\n Parallel Interface");
   printf ("\r\n IIF Modus");
   printf ("\r\n PIC18F14K22 Vers 5.1") ;
   delay_ms (2000);
*/

while(true){
   output_high(Clk); delay_ms(10);
   output_low(Clk); delay_ms(100);
   if ((rpos^wpos)>1) {
       i=read_uart();
       if ((i==0xC1)||(i==0xC5)||(i==0xC9)) {
           SR_out(read_uart());
           delay_ms(10);
           putc(SR_in()); }
       if (i==0xC5) {
           set_adc_channel( _EX );
           delay_us(100);
           EX=read_adc();
           putc(hiEX); putc(loEX); }
       if (i==0xC9) {
           set_adc_channel( _EY );
           delay_us(100);
           EY=read_adc();
           putc(hiEY); putc(loEY); }
       rpos=wpos;            //sync    
       }
   }
}

// ******** UART ISR  lesen und in Buffer schreiben ***************
#int_ext1
void uart_isr(void){ wpos=(wpos+1)&0x03; st[wpos]= getc();}
